"""
This type stub file was generated by pyright.
"""

ATEXT = ...
ATEXT_RE = ...
DOT_ATOM_TEXT = ...
ATEXT_INTL = ...
ATEXT_INTL_RE = ...
DOT_ATOM_TEXT_INTL = ...
ATEXT_HOSTNAME_INTL = ...
HOSTNAME_LABEL = ...
DOT_ATOM_TEXT_HOSTNAME = ...
DOMAIN_NAME_REGEX = ...
DOMAIN_LITERAL_CHARS = ...
QUOTED_LOCAL_PART_ADDR = ...
QTEXT_INTL = ...
EMAIL_MAX_LENGTH = ...
LOCAL_PART_MAX_LENGTH = ...
DNS_LABEL_LENGTH_LIMIT = ...
DOMAIN_MAX_LENGTH = ...
CASE_INSENSITIVE_MAILBOX_NAMES = ...
